@@ -1,5 +1,5 @@
 /* Copyright (C) 1995, 1996, 1997, 2000, 2001, 2003, 2004, 2006, 2008,
- *   2009, 2010, 2011, 2012, 2014 Free Software Foundation, Inc.
+ *   2009, 2010, 2011, 2012, 2014, 2015 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -47,7 +47,9 @@ extern double floor();
 
 #define SCM_MIN(A, B) ((A) < (B) ? (A) : (B))
 
-unsigned long 
+#if SCM_ENABLE_DEPRECATED == 1
+
+unsigned long
 scm_string_hash (const unsigned char *str, size_t len)
 {
   /* from suggestion at: */
@@ -59,15 +61,32 @@ scm_string_hash (const unsigned char *str, size_t len)
   return h;
 }
 
-unsigned long 
+#endif
+
+unsigned long
 scm_i_string_hash (SCM str)
 {
   size_t len = scm_i_string_length (str);
   size_t i = 0;
-
   unsigned long h = 0;
-  while (len-- > 0)
-    h = (unsigned long) scm_i_string_ref (str, i++) + h * 37;
+  const void *data;
+
+  data = scm_i_string_data (str);
+
+  if (scm_i_is_narrow_string (str))
+    {
+      const unsigned char *ndata = data;
+
+      for (i = 0; i < len; i++)
+	h = (unsigned long) ndata[i] + h * 37;
+    }
+  else
+    {
+      const scm_t_wchar *wdata = data;
+
+      for (i = 0; i < len; i++)
+	h = (unsigned long) wdata[i] + h * 37;
+    }
 
   scm_remember_upto_here_1 (str);
   return h;
